/* XMLAttibutes.java - Copyright (c) 2000 eXcelon Corporation, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of eXcelon
 * Corporation, Inc. ("Confidential Information").  You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with eXcelon Corporation, Inc.
 *
 * EXCELON CORP MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. EXCELON SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR 
 * ITS DERIVATIVES.
 */
 package com.exceloncorp.XML;

//import java.util.Collection;
import java.util.*;

/**
 * XMLAttributes defines the structure by which attributes for a
 * mark up language can be represented, especially those of the
 * XML standard.
 * 
 * @author David Millman
 * @version 1.0
 * @see com.exceloncorp.XML.XMLAttribute
 * @see com.exceloncorp.XML.XMLTag
 */



public class XMLAttributes implements OutputXML
{   

    /**
     * @param aName The name of the attribute to be added
     * @param aValue The value of the attribute to be added
     * @exception com.exceloncorp.XML.XMLInvalidAttributeNameException
     * @exception com.exceloncorp.XML.XMLInvalidAttributeValueException
     */
    //
    // Method(s)
    //
    public void addAttribute( String aName, String aValue ) throws com.exceloncorp.XML.XMLInvalidAttributeNameException,
                                                                   com.exceloncorp.XML.XMLInvalidAttributeValueException,
                                                                   com.exceloncorp.XML.XMLDuplicateAttributeException
    {
        XMLAttribute value;
        if ( aName != null )
        {
            value = getAttribute(aName);
            if ( value == null )
            {
                value = new XMLAttribute( aName, aValue );
                if ( isEmpty() == true )
                {
                    _attribute = value;

                }
                else if ( size() == ONE_ATTRIBUTE )
                {
                    _attributes = new Vector();
                    _attributes.add(_attribute);
                    _attributes.add(value);
                    _attribute = null;
                }
                else // more than one attribute is currently stored
                {
                    _attributes.add(value);
                    if ( _keyedAttributes != null )
                    {
                        _keyedAttributes.put(aName,value);
                    }
                    else if ( size() >= ADD_INDEX_CARDINALITY )
                    {
                        //
                        // Create the index and add all the values to the index.
                        //
                        _keyedAttributes = new HashMap();
                        XMLAttribute currentElement;
                        for ( int i=0; i < _attributes.size(); i++)
                        {
                            currentElement = (XMLAttribute)_attributes.elementAt(i);
                            _keyedAttributes.put(currentElement.getName(), currentElement);
                        }
                    }
                }
            }
            else
            {
                com.exceloncorp.XML.XMLDuplicateAttributeException duplicateException = new XMLDuplicateAttributeException(aName);
                throw duplicateException;
            }
        }
    } 
    
    public void setAttribute( String aName, String aValue ) throws com.exceloncorp.XML.XMLInvalidAttributeNameException,
                                                                   com.exceloncorp.XML.XMLInvalidAttributeValueException
    {
        try
        {
            this.addAttribute(aName, aValue);
        }
        catch ( com.exceloncorp.XML.XMLDuplicateAttributeException e )
        {
            XMLAttribute currentAttr = this.getAttribute(aName);
            currentAttr.setValue(aValue);
        }
    }
    
    public void print( XMLOutputStream aStream )
    {
        if (_attributes != null )
        {
            XMLAttribute currentAttr;
            for ( int currentAttribute = 0; currentAttribute < _attributes.size(); currentAttribute++)
            {
                aStream.add(ATTRIBUTE_SPACER);
                currentAttr = (XMLAttribute)_attributes.get(currentAttribute);
                currentAttr.print(aStream);
            }
        }
        else if ( _attribute != null )
        {
            aStream.add(ATTRIBUTE_SPACER);
            _attribute.print(aStream);
        }
    }
    
    protected int size()
    {
        int returnValue = NO_ATTRIBUTES;
        if ( isEmpty() == false )
        {
            if ( _attribute != null )
            {
                returnValue = ONE_ATTRIBUTE;
            }
            else
            {
                returnValue = _attributes.size();
            }
        }
        return returnValue;
    }
    
    
    private XMLAttribute getAttribute( String aValue )
    {
        XMLAttribute returnValue = null;
        if ( isEmpty() == false )
        {
            if ( _keyedAttributes != null )
            {
                returnValue = (XMLAttribute)_keyedAttributes.get(aValue);
            }
            else if ( _attribute != null ) // One attribute is valid
            {
                if ( _attribute.getName().compareTo(aValue) == 0 )
                {
                    returnValue = _attribute;
                }
            }
            else  // More than one attribute exists, but no index currently is available
            {
                int numberOfAttributes = _attributes.size();
                int currentElement = 0;
                XMLAttribute currentAttribute = null;
                while (( returnValue == null ) &&
                       ( currentElement < numberOfAttributes))
                {
                    currentAttribute = (XMLAttribute)_attributes.elementAt(currentElement);
                    if ( currentAttribute.getName().compareTo(aValue) == 0 )
                    {
                        returnValue = currentAttribute;
                    }
                    currentElement++;
                }
            }
        }
        return returnValue;
    }
    
    private XMLAttribute getAttribute( int aIndex )
    {
        XMLAttribute returnValue = null;
        return returnValue;
    }
    
    private boolean isEmpty()
    {
        boolean returnValue = false;
        if (( _attribute == null ) &&
            (_attributes == null ))
        {
            returnValue = true;
        }
        return returnValue;
    }
    
    //
    // Attribute(s)
    //
    XMLAttribute _attribute;
    Vector       _attributes;      // List of all the attribute(s)
    HashMap      _keyedAttributes;
    
    //
    // Constant(s)
    //
    private static final String ATTRIBUTE_SPACER      = " ";
    private static final int    NO_ATTRIBUTES         = 0;
    private static final int    ONE_ATTRIBUTE         = 1;
    private static final int    ADD_INDEX_CARDINALITY = 5;
}
