package com.sonicsw.sts.java.utils;

import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * DateTime provides a set of methods for controlling dates and times.
 * 
 * @author D.J.Millman
 */

public class DateTime {

   /**
    * Take in a julian date and convert it into the ISO string format.
    * 
    * @param argTime
    *           the time to convert from
    * @return the resulting ISO formatted date
    */
   public static String asISODateTime(long argTime) {
      String returnValue = getISOFormat().format(new Date(argTime));
      return returnValue;
   }

   /**
    * Take in a julian date and retrieve the month as a String.
    * 
    * @param argTime
    *           the time to convert from
    * @return the Month as a String
    */
   public static String asMonthNameFormat(long argTime) {
      String returnValue = DateTime.getMonthNameFormat().format(
            new Date(argTime));
      return returnValue;
   }

   public static String asYearFormat(long argTime) {
      String returnValue = DateTime.getYearFormat().format(new Date(argTime));
      return returnValue;
   }

   public static String asDayFormat(long argTime) {
      String returnValue = DateTime.getDayFormat().format(new Date(argTime));
      return returnValue;
   }

   public static String asTimestamp(long argTime) {
      String returnValue = DateTime.getTimestampFormat().format(
            new Date(argTime));
      return returnValue;
   }

   public static String asXSDDate(Date argDate) {
      return DateTime.getYMDFormat().format(argDate);
   }

   public static String asCRISFormat(Date argDate) {
      return DateTime.getCRISFormat().format(argDate);
   }

   /**
    * Return the String into a date time object
    */
   public static Date fromISODateTime(String argDateTime) {
      Date returnValue = null;
      ParsePosition pos = new ParsePosition(0);
      returnValue = getISOFormat().parse(argDateTime, pos);
      return returnValue;
   }

   public static Date fromYMDFormat(String argDate) {
      Date returnValue = null;
      ParsePosition pos = new ParsePosition(0);
      returnValue = getYMDFormat().parse(argDate, pos);
      return returnValue;
   }

   public static Date fromCRISFormat(String argDate) {
      Date returnValue = null;
      ParsePosition pos = new ParsePosition(0);
      returnValue = getCRISFormat().parse(argDate, pos);
      return returnValue;
   }

   public static Date fromC2KLoadFormat(String argDate) {
      Date returnValue = null;
      ParsePosition pos = new ParsePosition(0);
      returnValue = getC2KLoadFormat().parse(argDate, pos);
      return returnValue;
   }

   private static SimpleDateFormat getC2KLoadFormat() {
      SimpleDateFormat returnValue = sC2KFormat_;
      if ( returnValue == null )
      {
         sC2KFormat_ = new SimpleDateFormat(YMD_FORMAT);
         returnValue = sC2KFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getCRISFormat() {
      SimpleDateFormat returnValue = sCRISFormat_;
      if ( returnValue == null )
      {
         sCRISFormat_ = new SimpleDateFormat(CRIS_FORMAT);
         returnValue = sCRISFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getISOFormat() {
      SimpleDateFormat returnValue = sISODateFormat_;
      if ( returnValue == null )
      {
         sISODateFormat_ = new SimpleDateFormat(ISO_DATE_FORMAT);
         returnValue = sISODateFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getMonthNameFormat() {
      SimpleDateFormat returnValue = sMonthNameFormat_;
      if ( returnValue == null )
      {
         sMonthNameFormat_ = new SimpleDateFormat(MONTH_NAME_FORMAT);
         returnValue = sMonthNameFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getYearFormat() {
      SimpleDateFormat returnValue = sYearFormat_;
      if ( returnValue == null )
      {
         sYearFormat_ = new SimpleDateFormat(YEAR_FORMAT);
         returnValue = sYearFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getDayFormat() {
      SimpleDateFormat returnValue = sDayFormat_;
      if ( returnValue == null )
      {
         sDayFormat_ = new SimpleDateFormat(DAY_FORMAT);
         returnValue = sDayFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getYMDFormat() {
      SimpleDateFormat returnValue = sYMDFormat_;
      if ( returnValue == null )
      {
         sYMDFormat_ = new SimpleDateFormat(YMD_FORMAT);
         returnValue = sYMDFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat getTimestampFormat() {
      SimpleDateFormat returnValue = sTimestampFormat_;
      if ( returnValue == null )
      {
         sTimestampFormat_ = new SimpleDateFormat(TIMESTAMP_FORMAT);
         returnValue = sTimestampFormat_;
      }
      return returnValue;
   }

   private static SimpleDateFormat sISODateFormat_;

   private static SimpleDateFormat sMonthNameFormat_;

   private static SimpleDateFormat sYearFormat_;

   private static SimpleDateFormat sDayFormat_;

   private static SimpleDateFormat sYMDFormat_;

   private static SimpleDateFormat sTimestampFormat_;

   private static SimpleDateFormat sCRISFormat_;

   private static SimpleDateFormat sC2KFormat_;

   //
   // Constant(s)
   //
   private static final String     ISO_DATE_FORMAT   = "yyyy'-'MM'-'dd'T'hh:mm:ss.SSS";

   private static final String     MONTH_NAME_FORMAT = "MMMM";

   private static final String     YEAR_FORMAT       = "yyyy";

   private static final String     DAY_FORMAT        = "dd";

   private static final String     YMD_FORMAT        = "yyyy'-'MM'-'dd";

   private static final String     TIMESTAMP_FORMAT  = "yyyyMMdd';'hh':'mm':'ss':'SSS";

   private static final String     CRIS_FORMAT       = "yyyyMMdd";

   public static void main(String[] args) {
      long time = System.currentTimeMillis();
      System.out.println("Timestamp = " + DateTime.asTimestamp(time));

      /*
       * System.out.println("Time = " + Long.toString(time)); String val =
       * DateTime.asISODateTime(time); System.out.println("ISO date = " + val);
       * Date date = DateTime.fromISODateTime(val); System.out.println("Time = " +
       * Long.toString(date.getTime())); System.out.println("Year = " +
       * DateTime.asYearFormat(time)); System.out.println("Month = " +
       * DateTime.asMonthNameFormat(time)); System.out.println("Day = " +
       * DateTime.asDayFormat(time));
       */
   }

}