package com.sonicsw.sts.java.utils;

import java.util.Random;

public class OSRandom {

   /**
    * Create an integer based on the random object that is passed in, the value
    * returned would be within the specified range.
    * 
    * @param argRandom
    *           the random class used to create the number
    * @param argStartRange
    *           the start of the range
    * @param argEndRange
    *           the end of the range of valid numbers
    * @return the random number generated
    */
   public static int nextInt(Random argRandom, int argStartRange,
         int argEndRange) {
      int returnValue = argStartRange;
      if ( argRandom == null ) throw new NullPointerException(EXCEPT_MESG_1);
      if ( argStartRange > argEndRange )
            throw new RuntimeException(EXCEPT_MESG_2);
      int zeroedRange = (argEndRange - argStartRange);
      returnValue = argRandom.nextInt(zeroedRange) + argStartRange;
      return returnValue;
   }

   /**
    * Retrieve the next long random number, based on the Random class passed in,
    * the value will be between argStartRange and argEndRange.
    * 
    * @param argRandom
    *           the random class used to create the number
    * @param argStartRange
    *           the start of the range
    * @param argEndRange
    *           the end of the range of valid numbers
    * @return the random number generated
    */
   public static long nextLong(Random argRandom, long argStartRange,
         long argEndRange) {
      long returnValue = argStartRange;
      if ( argRandom == null ) throw new NullPointerException(EXCEPT_MESG_1);
      long range = argEndRange - argStartRange;
      if ( argStartRange > argEndRange )
            throw new RuntimeException(EXCEPT_MESG_2);
      returnValue = (returnValue % range) + argStartRange;
      return returnValue;
   }

   /**
    * Retrieve the next random number.
    * 
    * @param argStartRange
    *           the lowest bound of the range
    * @param argEndRange
    *           the highest bound of the range
    * @return the generated number
    */
   public static long nextLong(long argStartRange, long argEndRange) {
      long returnValue = argStartRange;
      if ( argStartRange > argEndRange )
            throw new RuntimeException(EXCEPT_MESG_2);
      long range = argEndRange - argStartRange;
      returnValue = (returnValue % range) + argStartRange;
      return returnValue;
   }

   /**
    * Return the next integer in a range
    */
   public static int nextInt(int argStartRange, int argEndRange) {
      int returnValue = argStartRange;
      if ( argStartRange > argEndRange )
            throw new RuntimeException(EXCEPT_MESG_2);
      argEndRange++;
      int zeroedRange = (argEndRange - argStartRange);
      returnValue = getRandom().nextInt(zeroedRange) + argStartRange;
      return returnValue;
   }

   /**
    * Return the random object that is used to generate the number
    */
   private static Random getRandom() {
      Random returnValue = sRandom_;
      if ( returnValue == null )
      {
         returnValue = new Random(System.currentTimeMillis());
         sRandom_ = returnValue;
      }
      return returnValue;
   }

   //
   // Attribute(s)
   //
   private static Random       sRandom_;

   //
   // Constant(s)
   //
   private static final String EXCEPT_MESG_1 = "No Random instance passed into the method";

   private static final String EXCEPT_MESG_2 = "Start of the range is greater than the end";

   public static void main(String[] args) {
      try
      {
         for (int i = 0; i < 100; i++)
         {
            System.out.println(nextInt(0, 10));
         }
      }
      catch (Throwable t)
      {
         t.printStackTrace();
      }
   }
}