/*
 * OSString.java - Copyright (c) 2000 eXcelon Corporation, Inc. All Rights
 * Reserved. This software is the confidential and proprietary information of
 * eXcelon Corporation, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with eXcelon
 * Corporation, Inc. EXCELON CORP MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. EXCELON SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package com.sonicsw.sts.java.utils;

/**
 * OSString provides a set of utilities to extend the normal java.langString
 * class, such as find and replace.
 * 
 * @author David Millman
 * @version 1.0
 */

public class OSString {

   /**
    * Replaces all the instances of find and replaces them with replaceWith.
    * 
    * @param string
    *           The string that is to changed with the new value
    * @param find
    *           The substring to be replaced in the input string
    * @param replaceWith
    *           The string that will be inserted for every instance of find
    *           found.
    * @return The updated string
    */
   public static String findAndReplace(String string, String find,
         String replaceWith) {
      StringBuffer returnValue = new StringBuffer();
      if ( (string != null) && (string.length() > 0) )
      {
         int currentIndex = 0;
         int previousIndex = 0;
         int findLength = find.length();
         int replaceLength = replaceWith.length();
         boolean found = true;

         while (found == true)
         {
            currentIndex = string.indexOf(find, currentIndex);
            if ( currentIndex < 0 )
            {
               found = false;
            }
            else
               if ( currentIndex >= 0 )
               {
                  returnValue.append(string.substring(0, currentIndex));
                  returnValue.append(replaceWith);
                  returnValue.append(findAndReplace(string
                        .substring(currentIndex + findLength), find,
                        replaceWith));
                  found = false;
               }
            currentIndex++;
         }
      }
      if ( returnValue.length() == EMPTY_STRING )
      {
         return string;
      }
      else
      {
         return returnValue.toString();
      }
   }

   /**
    * Returns the number of instances of a particular string that has been found
    * within another string.
    * 
    * @param argString
    *           the string in which to find the substring
    * @param argFind
    *           the string to find
    * @return the number of instances of argFind found.
    */
   public static int numberOfInstancesOf(String argString, String argFind) {
      int returnValue = 0;
      boolean found = true;
      int currentIndex = 0;
      int totalFound = 0;
      while (found == true)
      {
         currentIndex = argString.indexOf(argFind, currentIndex);
         if ( currentIndex < 0 )
         {
            found = false;
         }
         else
         {
            // An element has been found
            totalFound++;
            currentIndex++;
         }
      }
      returnValue = totalFound;
      return returnValue;
   }

   /**
    * Predicate to determine if the string passed in is valid, i.e not null and
    * that the length of the string is greater than 1.
    */
   public static boolean isValid(String argString) {
      boolean returnValue = false;
      if ( argString != null )
      {
         if ( argString.length() >= MINIMUM_LENGTH )
         {
            returnValue = true;
         }
      }
      return returnValue;
   }

   /**
    * Test to see if the string passed in is empty or equivalent, i.e. the
    * string reference is null.
    * 
    * @param argString
    *           the string to test
    * @return true if the string is null or 0 length
    */
   public static boolean isEmpty(String argString) {
      boolean returnValue = false;
      if ( (argString == null) || (argString.length() == EMPTY_STRING) )
      {
         returnValue = true;
      }
      return returnValue;
   }

   /**
    * Predicate that determines if the string, argOriginalString contains any of
    * the strings passed through by the argCOntainedString argument, if it does
    * then true is returned.
    * 
    * @param argOriginalString
    *           the string that is read to see if it contains any in the array
    *           of strings.
    * @param argContainedString
    *           an array of strings that are checked for containment in the
    *           original string
    * @return true if argOriginalString contains any string in
    *         argContainedString otherwise false
    */
   public static boolean containsString(String argOriginalString,
         String[] argContainedString) {
      boolean returnValue = false;
      int currentIndex = 0;
      while ((returnValue == false)
            && (currentIndex < argContainedString.length))
      {
         if ( argOriginalString.indexOf(argContainedString[currentIndex]) >= NO_MATCH )
         {
            returnValue = true;
         }
         currentIndex++;
      }
      return returnValue;
   }
   
   
   public static boolean isValidValueNoCase( String argOriginalString, String[] argValidValues){
	   boolean returnValue = false;
	   for ( int i=0; i < argValidValues.length; i++){
		   if ( argOriginalString.compareToIgnoreCase(argValidValues[i])== 0){
			   returnValue = true;
			   i = argValidValues.length + 1;
		   }
	   }
	   return returnValue;
   }
   
   public static int indexOfValidStringNoCase( String argOriginalString, String[] argValidValues ){
	   int returnValue = NO_MATCH;
	   for ( int i=0; i < argValidValues.length; i++){
		   if ( argOriginalString.compareToIgnoreCase(argValidValues[i])== 0){
			   returnValue = i;
			   i = argValidValues.length + 1;
		   }
	   }
	   return returnValue;
   }
   
   /**
    * Convert the array of strings that are passed in to a comma separated list of values
    * @param argValues the values to output
    * @return the response
    */
   public static String getStringArrayAsCSL( String[] argValues ){
	   StringBuffer returnValue = new StringBuffer();
	   for ( int i=0; i < argValues.length; i++){
		   if ( i > 0){
			   returnValue.append(",");
		   }
		   String currentValue = argValues[i];
		   returnValue.append(currentValue);
	   }	   
	   return returnValue.toString();
   }

   //
   // Constant(s)
   //
   public static final int NO_MATCH       = -1;

   public static final int EMPTY_STRING   = 0;

   public static final int MINIMUM_LENGTH = 1;

}