/*
 * XMLAttribute.java - Copyright (c) 2000 eXcelon Corporation, Inc. All Rights
 * Reserved. This software is the confidential and proprietary information of
 * eXcelon Corporation, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with eXcelon
 * Corporation, Inc. EXCELON CORP MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. EXCELON SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package com.sonicsw.sts.xmlstream;

import com.sonicsw.sts.java.utils.OSBoolean;

/**
 * XMLAttribute manages the output of an XML attribute.
 */

public class XMLAttribute implements OutputXML {
   //
   // Constructor(s)
   //

   /**
    * Construct an XML attribute
    */
   XMLAttribute(String aName, String aValue)
         throws com.sonicsw.sts.xmlstream.XMLInvalidAttributeNameException,
         com.sonicsw.sts.xmlstream.XMLInvalidAttributeValueException {
      if ( aName != null )
      {
         if ( com.sonicsw.sts.xmlstream.XMLAttributeName
               .isValidXMLAttributeName(aName) == true )
         {
            _name = new String(aName);
            if ( aValue != null )
            {
               if ( com.sonicsw.sts.xmlstream.XMLAttributeValue
                     .isValidXMLAttributeValue(aValue) == true )
               {
                  _value = new String(aValue);
               }
               else
               {
                  _value = new String(aValue);
               }
            }
         }
         else
         {
            XMLInvalidAttributeNameException e = new com.sonicsw.sts.xmlstream.XMLInvalidAttributeNameException();
            throw e;
         }
      }
   }

   /**
    * Create an XML attribute with a numeric value.
    * 
    * @param argName
    *           the name of the attribute
    * @param argValue
    *           the numeric value of the attribute
    */
   XMLAttribute(String argName, int argValue) {
      this(argName, Integer.toString(argValue));
   }

   /**
    * Create an XML Attribute with a boolean value
    * 
    * @param argName
    *           the name of the attribute
    * @param argBoolean
    *           the value of the attribute
    */
   XMLAttribute(String argName, boolean argBoolean) {
      this(argName, OSBoolean.booleanAsString(argBoolean));
   }

   //
   // Method(s)
   //

   /**
    * Write the attribute to the output stream
    * 
    * @param aStream
    *           the stream where the attribute is to be written out to
    */
   public void print(XMLOutputStream aStream) {
      if ( _name != null )
      {
         aStream.add(_name + XML_ATTRIBUTE_VALUE_EQUALS
               + XML_ATTRIBUTE_VALUE_DELIMITER);
         if ( _value != null )
         {
            aStream.add(_value);
         }
         aStream.add(XML_ATTRIBUTE_VALUE_DELIMITER);
      }
   }

   /**
    * Write the attribute as a String value
    * 
    * @return the string value
    */
   public String toString() {
      StringBuffer returnValue = new StringBuffer(getName()
            + XML_ATTRIBUTE_SPACER);
      returnValue.append(XML_ATTRIBUTE_VALUE_DELIMITER);
      if ( _value != null )
      {
         returnValue.append(this._value);
      }
      returnValue.append(XML_ATTRIBUTE_VALUE_DELIMITER);
      return returnValue.toString();
   }

   /**
    * Retrieve the name of the attribute
    * 
    * @return the name of the attribute
    */
   protected String getName() {
      return _name;
   }

   /**
    * Set the value of the attribute
    * 
    * @param aValue
    *           the value of the attribute
    */
   protected void setValue(String aValue) {
      if ( aValue == null )
      {
         _value = null;
      }
      else
      {
         _value = new String(_value);
      }
   }

   //
   // Attribute(s)
   //
   String                        _name;                               // The

   // name of
   // the
   // attribute

   String                        _value;                              // The

   // value
   // of the
   // attribute

   //
   // Constant(s)
   //
   protected static final String XML_ATTRIBUTE_VALUE_DELIMITER = "\"";

   protected static final String XML_ATTRIBUTE_VALUE_EQUALS    = "=";

   private static final String   XML_ATTRIBUTE_SPACER          = " ";
}