package com.sonicsw.sts.xmlstream;

import java.util.Properties;


/**
 * Provides a common interface for converting Java objects into an XML output
 * format. The methods in this provide a means by which multiple views should be
 * attainable for one class
 * 
 * @author D.J.Millman
 * @version 1.0
 */

public interface XMLOutput {

   /**
    * Write the object out to the XML stream as decided by the properties passed
    * into the argProperties position. The return value of XMLOutput stream is
    * defined so that if no stream is passed in then it is possible for one to
    * be created and passed back.
    * 
    * @param argStream
    *           the stream that the XML content is to be added to, or null if no
    *           stream is being passed in and one needs to be created.
    * @param argProperties
    *           the properties that describe how the object will render itself,
    *           this should never be null.
    * @return the output stream that is passed in or the newly created one if
    *         this has been done.
    */
   public XMLOutputStream toXML(com.sonicsw.sts.xmlstream.XMLOutputStream argStream,
         Properties argProperties) throws XMLOutputStreamException;

   //
   // Property Constant(s)
   //
   // This is the set of standard property labels that should be used.
   //

   /**
    * If the output should use a namespace then the namespace should be
    * specified in the NAMESPACE_PROPERTY
    */
   public static final String NAMESPACE_PROPERTY = "namespace";

   /**
    * Typically objects can generate multiple named views, the property name
    * VIEW_PROPERTY specifies a view that the object should represent itself
    * into.
    */
   public static final String VIEW_PROPERTY      = "view";
}