package com.sonicsw.sts.xmlstream;

import org.w3c.dom.Document;
import org.w3c.dom.Node;


/**
 * XMLOutputStreamFactory is responsible for creating an XMLOutputStream that
 * can be written to to create formatted XML output.
 * 
 * @author D.J.Millman
 * @version 1.0
 */

public class XMLOutputStreamFactory {

   /**
    * Return an OutputStream that writes to a string
    * 
    * @return the XMLOutputStream that writes it's data to a string.
    */
   public static XMLOutputStream create() throws Exception {
      return create(null);
   }
   
   public static XMLOutputStream create( Node argNode, boolean argFromRootNode ) throws Exception{
	   XMLOutputStream returnValue = null;
	   if ( argNode == null ) throw new XMLOutputStreamException("The output node cannot be null");
	   Node startNode = null;
	   if ( argNode.getNodeType() == Node.DOCUMENT_NODE){
		   startNode = argNode;
		   if ( startNode == null ) throw new Exception("No document found");
		   returnValue = create();
		   returnValue.startDocument();
		   Node rootNode = startNode.getFirstChild();
		   returnValue.addXMLData(rootNode);
		   returnValue.endDocument();		   
	   }
	   else if ( argFromRootNode == true ){
		   startNode = argNode.getOwnerDocument();
		   if ( startNode == null ) throw new Exception("No document found");
		   returnValue = create();
		   returnValue.startDocument();
		   Node rootNode = startNode.getFirstChild();
		   returnValue.addXMLData(rootNode);
		   returnValue.endDocument();
	   }
	   else{
		   if ( argNode.getNodeType() == Node.TEXT_NODE){
			   throw new XMLOutputStreamException("The output node cannot start at a text node");
		   }
		   startNode = argNode;
		   returnValue = create();
		   returnValue.startDocument();
		   returnValue.addXMLData(startNode);
		   returnValue.endDocument();
		   
	   }
	   return returnValue;
   }

   /**
    * Create an XMLOutputStream and return it to the caller.
    * 
    * @param argFilename
    *           the name of the file to create, if the value is null then a text
    *           string is created rather than a file.
    * @return the XMLOutputStream that will be written to or null if it cannot
    *         be used.
    */
   public static com.sonicsw.sts.xmlstream.XMLOutputStream create(String argFilename)
         throws Exception {
      com.sonicsw.sts.xmlstream.XMLOutputStream returnValue = null;
      try
      {
         if ( argFilename == null )
         {
            returnValue = new com.sonicsw.sts.xmlstream.XMLOutputStreamImpl();
         }
         else
         {
            returnValue = new com.sonicsw.sts.xmlstream.XMLOutputStreamImpl(
                  argFilename);
         }
      }
      catch (Exception e)
      {
         throw e;
      }
      return returnValue;
   }

   /**
    * Create an XMLOutputStream and write it to the file specified if the file
    * already exists then it is overwritten and the old file is written to a
    * point dependent on the current versioning strategy.
    * 
    * @param argFilename
    *           the name of the file to create and write to
    * @param argVersionable
    *           specifiy whether the to using version control.
    * @return the created XMLOutputStream or null if it does not exist
    */
   public static com.sonicsw.sts.xmlstream.XMLOutputStream create(String argFilename,
         boolean argVersionable) throws Exception {
      return null;
   }

   public static void main(String[] args) {
      try
      {
         XMLOutputStream stream = XMLOutputStreamFactory.create(null);
         XMLTagMarker tags = stream.getTagMarker();
         stream.startTag("DavesTag");
         stream.closeTags(tags);
         System.out.println("Output Stream " + stream.toString());
      }
      catch (Exception e)
      {
         System.out.println(e);
         e.printStackTrace();
      }
   }
}