/*
 * XMLTagName.java - Copyright (c) 2000 eXcelon Corporation, Inc. All Rights
 * Reserved. This software is the confidential and proprietary information of
 * eXcelon Corporation, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in accordance
 * with the terms of the license agreement you entered into with eXcelon
 * Corporation, Inc. EXCELON CORP MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. EXCELON SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package com.sonicsw.sts.xmlstream;

public class XMLTagName implements OutputXML {
   //
   // Constructor(s)
   //
   public XMLTagName(String aNamespace, String aTagName)
         throws com.sonicsw.sts.xmlstream.XMLInvalidTagNameException,
         com.sonicsw.sts.xmlstream.XMLInvalidNamespaceException {
      this(aTagName);
      if ( aNamespace != null )
      {
         if ( isValidXMLNamespaceName(aNamespace) == true )
         {
            _namespace = new String(aNamespace);
         }
         else
         {
            com.sonicsw.sts.xmlstream.XMLInvalidNamespaceException e = new com.sonicsw.sts.xmlstream.XMLInvalidNamespaceException();
            throw e;
         }
      }
   }

   public XMLTagName(String aTagName)
         throws com.sonicsw.sts.xmlstream.XMLInvalidTagNameException {
      if ( isValidXMLTagName(aTagName) == true )
      {
         _tagname = new String(aTagName);
      }
      else
      {
         throw (new com.sonicsw.sts.xmlstream.XMLInvalidTagNameException());
      }
   }

   //
   // Method(s)
   //
   public void print(XMLOutputStream aStream) {
      aStream.add("</");
      if ( _namespace != null )
      {
         aStream.add(_namespace + ":");
      }
      aStream.add(_tagname + ">");
   }

   protected boolean isNamespaceDefined() {
      boolean returnValue = false;
      if ( _namespace != null )
      {
         returnValue = true;
      }
      return returnValue;
   }

   protected String getNamespace() {
      return _namespace;
   }

   protected String getTagName() {
      return _namespace;
   }

   //
   // Attribute(s)
   //
   private String _namespace;

   private String _tagname;

   //
   // Method(s)
   //
   public static boolean isValidXMLTagName(String aName) {
      boolean returnValue = false;
      if ( aName != null )
      {
         java.util.Vector invalidValues = XMLTagName.invalidValues();
         int invalidValueSize = invalidValues.size();
         if ( invalidValueSize > EMPTY_INVALID_VALUES_COLLECTION )
         {
            returnValue = true;
            int currentIndex = 0;
            while ((returnValue == true) && (currentIndex < invalidValueSize))
            {
               if ( aName.indexOf((String) invalidValues
                     .elementAt(currentIndex)) != -1 )
               {
                  returnValue = false;
               }
               currentIndex++;
            }
         }
      }
      return returnValue;
   }

   public static boolean isValidXMLNamespaceName(String aName) {
      boolean returnValue = true;
      if ( aName != null )
      {
         java.util.Vector invalidValues = XMLTagName.invalidValues();
         int invalidValueSize = invalidValues.size();
         if ( invalidValueSize > EMPTY_INVALID_VALUES_COLLECTION )
         {
            returnValue = true;
            int currentIndex = 0;
            while ((returnValue == true) && (currentIndex < invalidValueSize))
            {
               if ( aName.indexOf((String) invalidValues
                     .elementAt(currentIndex)) != -1 )
               {
                  returnValue = false;
               }
               currentIndex++;
            }
         }
      }
      return returnValue;
   }

   /**
    * Returns a list of values that are invalid in an XML tag name.
    */

   private static java.util.Vector invalidValues() {
      java.util.Vector returnValue = null;
      if ( _vectorInvalidValues == null )
      {
         _vectorInvalidValues = new java.util.Vector();
         _vectorInvalidValues.addElement(new String(" "));
         _vectorInvalidValues.addElement(new String(":"));
      }
      returnValue = _vectorInvalidValues;
      return returnValue;
   }

   private static java.util.Vector _vectorInvalidValues;

   private static final int        EMPTY_INVALID_VALUES_COLLECTION = 0;

}